package de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.analyzer;

import de.tudarmstadt.maki.simonstrator.api.common.UniqueID;
import de.tudarmstadt.maki.simonstrator.api.component.core.MonitorComponent;

/**
 * Analyzer for the distribution of resources. Records the maximum amount of resources and is updated whenever
 * resources are removed from a distributor.
 *
 * @author Tim Feuerbach
 */
public interface ResourceDistributionAnalyzer extends MonitorComponent.Analyzer {
    /**
     * Adds to the maximum amount of resources that will be distributed during the simulation. This method will be
     * called once or multiple times at the beginning of the simulation; afterwards, the maximum amount is fixed.
     * <p/>
     * Implementors: Keep track of this value and decrease when {@link #resourceAmountChanged(UniqueID, int, int)}
     * indicates a fill up of resources to find the moment when no more resources are available.
     */
    void addMaxResources(short resourceType, int resourceAmount);

    /**
     * Must be called if a distributor is removed from the field while still holding resources, and these resources
     * were added back to the global pool of distributable resources.
     */
    void restoreMaxResources(UniqueID beaconID, int resourceAmount);

    /**
     * Called when the amount of resources at the given beacon changes. What caused the change depends on the number
     * before and after: If the amount decreased, a consumer took resources. If the number increased, resources were
     * refilled or the distributor was moved and reused as a new resource spawn.
     * <p/>
     * The method <b>must</b> be called at the beginning by a distributor with the start amount as newResources and
     * 0 as oldResources. The total amount of resources distributed may not exceed the maximum value indicated by
     * {@link #addMaxResources(short, int)}.
     */
    void resourceAmountChanged(UniqueID beaconID, int oldResources, int newResources);
}
