package de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.analyzer;

import de.tudarmstadt.maki.simonstrator.api.common.UniqueID;
import de.tudarmstadt.maki.simonstrator.api.common.graph.INodeID;
import de.tudarmstadt.maki.simonstrator.api.component.core.MonitorComponent;
import de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.ResourceDistributionNotification;

/**
 * An analyzer to write out resource reception receipts and occasions when there was no suiting beacon.
 * <p/>
 * Measures the amount of time a
 * {@link de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.ResourceConsumer}
 * spent in
 * {@link de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.ResourceConsumptionState#HEADING_FOR_RESOURCE HEADING state}
 * when it receives resources or leaves the state because it was notified that all resources were taken.
 *
 * @author Tim Feuerbach
 */
public interface ResourceReceptionAnalyzer extends MonitorComponent.Analyzer {
    public static enum NoSuitingBeaconReason {
        /**
         * The node would use up a vital resource before arriving at the resource distributor.
         */
        MOVEMENT_TO_BEACON_WOULD_KILL_NODE,

        /**
         * There is no beacon known that has > 0 resources of the required type available.
         */
        NO_KNOWN_BEACON_WITH_AVAILABLE_RESOURCES,

        /**
         * There are resources available, but other nodes have already reserved their share via a negotiation
         * algorithm.
         */
        ALL_RESERVED,

        /**
         * Reserved for future use. The strategy actively decided to reject resource distributors that do
         * not have one of the other limitations.
         */
        ACTIVE_STRATEGY_DECISION,
    }

    /**
	 * Must be called when a node begins moving towards a resource distributor.
	 *
	 * @param consumer
	 *            the consumer node's ID
	 * @param distributor
	 *            the distributor's beacon ID
	 * @param advertisement
	 *            the target's advertisement
	 * @param resourceType
	 *            the resource type ID
	 * @param startTime
	 *            time when consumer started heading for resource (semantics of
	 *            {@link de.tudarmstadt.maki.simonstrator.api.Time})
	 * @see #visitedResourceDistributor(INodeID, UniqueID, short, double, long)
	 *      Method to call when the node arrives
	 * @see #canceledHeading(INodeID, UniqueID, short, long) Method to call when
	 *      the node cancels movement
	 */
	void startedHeading(INodeID consumer, UniqueID distributor, ResourceDistributionNotification advertisement,
			short resourceType,
			long startTime);

    /**
     * Must be called when a consumer arrived at a resource distributor and obtained 0 ore more resources.
     *
     * @param consumer          the consumer's ID
     * @param distributor       the distributor's beacon ID
     * @param resourceType      the resource type ID
     * @param resourcesReceived the amount of resources received; must be >= 0
     * @param timeStopped       time consumer stopped (semantics of {@link de.tudarmstadt.maki.simonstrator.api.Time})
     */
    void visitedResourceDistributor(INodeID consumer, UniqueID distributor, short resourceType, double resourcesReceived,
                                    long timeStopped);

    /**
     * Must be called when a consumer canceled moving towards a resource distributor.
     *
     * @param consumer     the consumer's ID
     * @param distributor  the distributor's beacon ID
     * @param resourceType the resource type ID
     * @param timeStopped  time consumer stopped (semantics of {@link de.tudarmstadt.maki.simonstrator.api.Time})
     */
    void canceledHeading(INodeID consumer, UniqueID distributor, short resourceType,
                         long timeStopped);

    /**
     * Must be called when a resource selection strategy could not find a beacon that fulfills the consumer's demand.
     *
     * @param consumer the consumer
     * @param reason   why there was no beacon available
     * @param time     time the strategy searched for a beacon
     */
    void couldNotFindBeaconForDemand(INodeID consumer, NoSuitingBeaconReason reason, long time);
}
