package de.tudarmstadt.maki.simonstrator.api.component.sensor.resource;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.SensorComponent;

/**
 * Provides the current number of resource units available to a host. Works
 * similarly to a
 * {@link de.tudarmstadt.maki.simonstrator.api.component.sensor.battery.BatterySensor}
 * .
 *
 * @author Tim Feuerbach
 * @author Julian Zobel
 */
public interface ResourceSensor extends SensorComponent {
    /**
     * Returns a list of resource type IDs that this sensor can monitor.
     */
    public short[] getResourceTypes();

    /**
     * Returns a name for the given resource type ID.
     *
     * @return null if the id is unknown to this sensor
     */
	public String getName(short resourceTypeId);

    /**
     * Returns the current number of resource units of the given resource type carried by the node.
     *
     * @param resourceType resource type ID
     * @throws IllegalArgumentException if the resource type is not monitored by this sensor
     * @see #getMaxResourceCount(short)
     */
	public double getResourceCount(short resourceTypeId);

    /**
     * Returns the maximum number of resource units the node can transport.
     *
     * @param resourceType resource type ID
     * @return maximum number of resources the node can transport; values below 0 mean unlimited
     * @throws IllegalArgumentException if the resource type is not monitored by this sensor
     * @see #getResourceCount(short)
     */
	public double getMaxResourceCount(short resourceTypeId);

    /**
     * Increases the number of stored resource units for the given resource type up to the maximum capacity.
     * <p/>
     * <b>This should only be called by the simulation or if resource fill up cannot be sensed (e.g., user has
     * to tell application manually)</b>
     *
     * @param resourceTypeId identifier of the resource you want to fill up
     * @param resourceUnits  a positive value
     * @throws IllegalArgumentException if the resource type ID has not been registered beforehand
     * @throws IllegalArgumentException if resourceUnits is below 0
     */
    public void _fillResourceUnits(short resourceTypeId, double resourceUnits);

	/**
	 * Sets the number of stored resource units for the given resource type ID
	 * to a certain value. The amount of resource units must be greater or equal
	 * to 0. If it is higher than the maximum storage capacity of the resource
	 * unit, it will be set to that maximum.
	 * 
	 * <b>This should only be called from the simulation for demonstration
	 * purposes</b>
	 * 
	 * @param resourceTypeId
	 *            identifier of the resource you want to fill up
	 * @param resourceUnits
	 *            a positive value between 0.0 and 1.0
	 * @throws IllegalArgumentException
	 *             if the resource type ID has not been registered beforehand
	 * @throws IllegalArgumentException
	 *             if resourceUnits is below 0
	 */
	public void setResourcePercentage(short resourceTypeId, double resourcePercentage);
}
